# Copyright (c) 2023, BLUEWALM. All rights reserved. 
# context

import contextlib
# 
from .embedding_replacement import embedding_replacement
from .initializer_replacement import initializer_replacement


@contextlib.contextmanager
def decomposed_embeddings(**kwargs):
    r""" Creates the Python context in which the `torch.nn.Embedding` class is overridden. 
       Any calls to initializers in `torch.nn.init` are also overridden. 
    
    kwArgs:
        embedding_type (list): a list of elements from {`native`, `frobenius`}. 
                            sets the type of embedding to be constructed, 
                            in the order in which embeddings are constructed. 
        frobenius_rank (list): a list of positive integers. each `fpd` layer pops one integer 
                          from this list at the time its constructor is being called. 
        frobenius_blocks (list): a list of positive integers. each `fpd` layer pops one integer 
                            from this list at the time its constructor is being called. 
        alternating_gradients (bool): setting it to True will result in a slower, 
                                but a more stable convergence. 
                                relevant to `frobenius` layers only. 
        onchip_memory (bool): setting it to True will result in a small latency drop, 
                            but it will not use any global memory. `frobenius` layers only. default: True 
        silent (bool): setting it to True will result in the details of the embedding hidden. 
                    hyperparameters, size, etc. will not be printed to console when `print(model)` 
                    is called, respectively. default: `False`
        sparse (bool): set it to True to use sparse embeddings. only relevant for `native` layers. default: `False`
        device (str): sets the device the embeddings will be constructed on. default: `cuda`
        dtype (torch.dtype): sets the datatype of the embeddings at construction time. 
                         supported: `torch.float32` and `torch.float16`. default: `torch.float32`
    """
    with initializer_replacement(**kwargs):
        with embedding_replacement(**kwargs):
            try:
                yield
            finally:
                pass

