# Copyright (c) 2023, BLUEWALM. All rights reserved. 
# embedding replacer

import torch
import contextlib
# 
import pytorch_embeddings.globals as globals
from .replacer import Replacer, entry
from .BaseEmbedding import BaseEmbedding
from .NativeEmbedding import NativeEmbedding
from .FrobeniusEmbedding import FrobeniusEmbedding


def activate(**outside_kwargs):
    class Embedding(BaseEmbedding):
        def __init__(self, *args, **kwargs):
            kwargs.update(outside_kwargs)
            super().__init__(*args, **kwargs)
    map_to_replacement = [entry(torch.nn, 'Embedding', Embedding)]
    _embedding_replacer.reset(map_to_replacement)
    _embedding_replacer.apply_replacements()


def deactivate():
    globals.callback_embeddings.clear()
    _embedding_replacer.apply_inverse_replacements()


def pop_frobenius_rank(frobenius_rank_list):
    if frobenius_rank_list:
        return frobenius_rank_list.pop()
    else:
        raise Exception("need one more frobenius-rank specified")


def pop_frobenius_blocks(frobenius_blocks_list):
    if frobenius_blocks_list:
        return frobenius_blocks_list.pop()
    else:
        raise Exception("need one more frobenius-blocks specified")


def callbacks(**kwargs):
    embedding_type_list = kwargs.get('embedding_type', [])
    frobenius_rank_list = kwargs.get('frobenius_rank', [])
    frobenius_rank_list.reverse()
    frobenius_blocks_list = kwargs.get('frobenius_blocks', [])
    frobenius_blocks_list.reverse()
    default = (0 == len(frobenius_blocks_list + frobenius_rank_list))
    silent = kwargs.get('silent', False)
    alternating_gradients = kwargs.get('alternating_gradients', False)
    onchip_memory = kwargs.get('onchip_memory', True)
    type_to_embedding = {'native' : NativeEmbedding, 
                        'frobenius' : FrobeniusEmbedding}
    config = []
    for embedding_type in embedding_type_list:
        embedding_config = {'silent' : silent, 
                           'alternating_gradients' : alternating_gradients, 
                           'onchip_memory' : onchip_memory}
        if embedding_type == 'native':
            pass
        elif (embedding_type == 'frobenius') and (not default):
            frobenius_rank = pop_frobenius_rank(frobenius_rank_list)
            frobenius_blocks = pop_frobenius_blocks(frobenius_blocks_list)
            embedding_config['frobenius_rank'] = frobenius_rank
            embedding_config['frobenius_blocks'] = frobenius_blocks
        embedding_config['embedding_type'] = type_to_embedding[embedding_type]
        config.append(embedding_config)
    # reset the embeddings #  
    for embedding, embedding_config in zip(globals.callback_embeddings, config):
        embedding.reset(**embedding_config)


# global object that maintains the state of the embedding replacements #  
_embedding_replacer = Replacer()


@contextlib.contextmanager
def embedding_replacement(**kwargs):
    kwargs = {k : v for k,v in kwargs.items() if v is not None}
    embedding_type_list = kwargs.get('embedding_type', [])
    if 0 < len(embedding_type_list):
        activate(**kwargs)
    try:
        yield
    finally:
        callbacks(**kwargs)
        deactivate()

