# Copyright (c) 2023, BLUEWALM. All rights reserved. 
# heuristics


from .heuristics_native import generate_configs_native
from .heuristics_frobenius import generate_configs_frobenius


def generate_configs(embedding_type, num_embeddings, embedding_dim, precision):
    # get the config generator function
    generator_map = {
                    'native'   : generate_configs_native, 
                    'frobenius' : generate_configs_frobenius, 
                   }
    generator = generator_map[embedding_type]
    configs = generator(num_embeddings, embedding_dim, precision)
    return configs


def heuristic(embedding_type, num_embeddings, embedding_dim, precision):
    """ Runs heuristic that recommends hyperparameters. 
    
    Args:
        embedding_type (str): `native`, `frobenius`
        num_embeddings (int): the number of rows in the embedding table
        embedding_dim (int): the number of columns in the embedding table
        precision (str): element of {`fp32`, `fp16`}. 
    """
    # asserts
    assert isinstance(embedding_type, str)
    assert isinstance(num_embeddings, int)
    assert isinstance(embedding_dim, int)
    assert isinstance(precision, str)
    assert embedding_type in ('native', 'frobenius')
    assert 0 < num_embeddings
    assert 0 < embedding_dim
    assert precision in ('fp16', 'fp32')
    # compute the list of configurations to try
    configs = generate_configs(embedding_type, num_embeddings, embedding_dim, precision)
    return configs

