# Copyright (c) 2023, BLUEWALM. All rights reserved. 
# initializer replacement

import torch
import contextlib
# 
import pytorch_embeddings.globals as globals
from .replacer import Replacer, entry
from .BaseEmbedding import BaseEmbedding


def replacement_generator(library_name, function_name):
    original_function = getattr(library_name, function_name)
    def replacement(*args, **kwargs):
        if (0 < len(args)) and isinstance(args[0], BaseEmbedding):
            base = args[0]
            args = list(args)[1:]
            arguments = (original_function, function_name, args, kwargs)
            initializer = (base.set_state_from_random_source, arguments)
            globals.callback_initializers.append(initializer)
        elif isinstance(kwargs.get('tensor', None), BaseEmbedding):
            base = kwargs.get('tensor')
            del kwargs['tensor']
            arguments = (original_function, function_name, args, kwargs)
            initializer = (base.set_state_from_random_source, arguments)
            globals.callback_initializers.append(initializer)
        else:
            return original_function(*args, **kwargs)
    return replacement


def activate():
    library = torch.nn.init
    initializers = [
                # supported
                'uniform_', 'normal_', 'trunc_normal_', 
                'constant_', 'ones_', 'zeros_', 
                'xavier_uniform_', 'xavier_normal_', 
                'kaiming_uniform_', 'kaiming_normal_', 
                'sparse_', 
                # unsupported
                'eye_', 'dirac_', 'orthogonal_'
                ]
    map_to_replacement = [entry(library, name, replacement_generator(library, name)) for name in initializers]
    _initializer_replacer.reset(map_to_replacement)
    _initializer_replacer.apply_replacements()


def deactivate():
    globals.callback_initializers.clear()
    _initializer_replacer.apply_inverse_replacements()


def callbacks():
    for initializer,arguments in globals.callback_initializers:
        initializer(*arguments)


# global object that maintains the state of the initializer replacements
_initializer_replacer = Replacer()


@contextlib.contextmanager
def initializer_replacement(**kwargs):
    kwargs = {k : v for k,v in kwargs.items() if v is not None}
    embedding_type_list = kwargs.get('embedding_type', [])
    if 0 < len(embedding_type_list):
        activate()
    try:
        yield
    finally:
        callbacks()
        deactivate()

