# Copyright (c) 2023, BLUEWALM. All rights reserved. 
# replacer

from collections import namedtuple


# definition of the named tuple that is used to store mapping of the replacements
entry = namedtuple('replacement_entry', 'library_name original replacement')


class Replacer:
    r''' helper class for replacing pytorch library classes and functions with custom defined ones '''
    def __init__(self, map_to_replacement=None):
        if map_to_replacement is None:
            map_to_replacement = []
        self.reset(map_to_replacement)
    
    def reset(self, map_to_replacement):
        self.map_to_replacement = map_to_replacement
        self.map_to_original = [entry(item[0], item[1], getattr(item[0], item[1])) for item in map_to_replacement]
    
    def apply_replacements(self):
        for item in self.map_to_replacement:
            setattr(item.library_name, item.original, item.replacement)
    
    def apply_inverse_replacements(self):
        for item in self.map_to_original:
            setattr(item.library_name, item.original, item.replacement)

